'use strict'

import castPath    from './castPath.js'
import isArguments from '../isArguments.js'
import isIndex     from '../isIndex.js'
import isLength    from '../isLength.js'
import toKey       from './toKey.js'

/**
 * Transform a float number into this upper integer representation.
 * @name hasPath
 * @memberof core.objects
 * @function
 * @instance
 * @example
 * const object = { 'a': { 'b': 2 } } ;
 *
 * trace( hasPath(object , 'a.b') ) ; // true
 * trace( hasPath(object , ['a', 'b']) ) ; // true
 */
const hasPath = ( object , path ) =>
{
    path = castPath( path , object ) ;

    let key ;
    let index  = -1 ;
    let { length }      = path ;
    let result = false ;

    while (++index < length)
    {
        key = toKey(path[index]);
        if (!(result = object !== null && Object.prototype.hasOwnProperty.call(object, key)))
        {
            break;
        }
        object = object[key] ;
    }

    if ( result || ++index !== length )
    {
        return result;
    }

    length = ( object === null || object === undefined ) ? 0 : object.length;

    return (
        !!length &&
        isLength(length) &&
        isIndex(key, length) &&
        (Array.isArray(object) || isArguments(object))
    );
}

export default hasPath ;